# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Initial version

Revision ID: 001
Revises: None
Create Date: 2016-08-18 12:28:37.372366

"""

# revision identifiers, used by Alembic.
revision = '001'
down_revision = None

from alembic import op
import sqlalchemy as sa


MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    op.create_table(
        'glare_artifacts',
        sa.Column('id', sa.String(36), primary_key=True, nullable=False),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('type_name', sa.String(255), nullable=False),
        sa.Column('version_prefix', sa.BigInteger(), nullable=False),
        sa.Column('version_suffix', sa.String(255)),
        sa.Column('version_meta', sa.String(255)),
        sa.Column('description', sa.Text()),
        sa.Column('visibility', sa.String(32), nullable=False),
        sa.Column('status', sa.String(32), nullable=False),
        sa.Column('owner', sa.String(255)),
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('activated_at', sa.DateTime()),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    op.create_index('ix_glare_artifact_name_and_version',
                    'glare_artifacts',
                    ['name', 'version_prefix', 'version_suffix']
                    )
    op.create_index('ix_glare_artifact_type',
                    'glare_artifacts',
                    ['type_name']
                    )
    op.create_index('ix_glare_artifact_status',
                    'glare_artifacts',
                    ['status']
                    )
    op.create_index('ix_glare_artifact_owner',
                    'glare_artifacts',
                    ['owner']
                    )
    op.create_index('ix_glare_artifact_visibility',
                    'glare_artifacts',
                    ['visibility']
                    )

    op.create_table(
        'glare_artifact_tags',
        sa.Column('id', sa.String(36), primary_key=True, nullable=False),
        sa.Column('artifact_id', sa.String(36),
                  sa.ForeignKey('glare_artifacts.id'), nullable=False),
        sa.Column('value', sa.String(255), nullable=False),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    op.create_index('ix_glare_artifact_tags_artifact_id',
                    'glare_artifact_tags',
                    ['artifact_id']
                    )
    op.create_index('ix_glare_artifact_tags_artifact_id_tag_value',
                    'glare_artifact_tags',
                    ['artifact_id', 'value']
                    )

    op.create_table(
        'glare_artifact_blobs',
        sa.Column('id', sa.String(36), primary_key=True, nullable=False),
        sa.Column('artifact_id', sa.String(36),
                  sa.ForeignKey('glare_artifacts.id'), nullable=False),
        sa.Column('size', sa.BigInteger()),
        sa.Column('md5', sa.String(32)),
        sa.Column('sha1', sa.String(40)),
        sa.Column('sha256', sa.String(64)),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('status', sa.String(32), nullable=False),
        sa.Column('external', sa.Boolean()),
        sa.Column('url', sa.Text()),
        sa.Column('key_name', sa.String(2048)),
        sa.Column('content_type', sa.String(255)),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    op.create_index('ix_glare_artifact_blobs_artifact_id',
                    'glare_artifact_blobs',
                    ['artifact_id']
                    )
    op.create_index('ix_glare_artifact_blobs_name',
                    'glare_artifact_blobs',
                    ['name']
                    )

    op.create_table(
        'glare_artifact_properties',
        sa.Column('id', sa.String(36), primary_key=True, nullable=False),
        sa.Column('artifact_id', sa.String(36),
                  sa.ForeignKey('glare_artifacts.id'), nullable=False),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('string_value', sa.String(20000)),
        sa.Column('int_value', sa.Integer()),
        sa.Column('numeric_value', sa.Numeric()),
        sa.Column('bool_value', sa.Boolean()),
        sa.Column('position', sa.Integer()),
        sa.Column('key_name', sa.String(255)),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    op.create_index('ix_glare_artifact_properties_artifact_id',
                    'glare_artifact_properties',
                    ['artifact_id']
                    )
    op.create_index('ix_glare_artifact_properties_name',
                    'glare_artifact_properties',
                    ['name']
                    )

    op.create_table(
        'glare_artifact_locks',
        sa.Column('id', sa.String(255), primary_key=True, nullable=False),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )


def downgrade():
    op.drop_table('glare_artifact_locks')
    op.drop_table('glare_artifact_properties')
    op.drop_table('glare_artifact_blobs')
    op.drop_table('glare_artifact_tags')
    op.drop_table('glare_artifacts')

    # end Alembic commands #



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import jsonschema

from glare.tests.functional import base

fixture_base_props = {
    u'activated_at': {
        u'description': u'Datetime when artifact has became active.',
        u'filter_ops': [u'lt', u'gt'],
        u'format': u'date-time',
        u'glareType': u'DateTime',
        u'readOnly': True,
        u'required_on_activate': False,
        u'sortable': True,
        u'type': [u'string',
                  u'null']},
    u'created_at': {
        u'description': u'Datetime when artifact has been created.',
        u'filter_ops': [u'lt', u'gt'],
        u'format': u'date-time',
        u'glareType': u'DateTime',
        u'readOnly': True,
        u'sortable': True,
        u'type': u'string'},
    u'description': {u'default': u'',
                     u'description': u'Artifact description.',
                     u'filter_ops': [],
                     u'glareType': u'String',
                     u'maxLength': 4096,
                     u'mutable': True,
                     u'required_on_activate': False,
                     u'type': [u'string',
                               u'null']},
    u'id': {u'description': u'Artifact UUID.',
            u'filter_ops': [u'eq',
                            u'neq',
                            u'in'],
            u'glareType': u'String',
            u'maxLength': 255,
            u'pattern': u'^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}'
                        u'-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$',
            u'readOnly': True,
            u'sortable': True,
            u'type': u'string'},
    u'metadata': {u'additionalProperties': {u'maxLength': 255,
                                            u'minLength': 1,
                                            u'type': u'string'},
                  u'default': {},
                  u'description': u'Key-value dict with useful information '
                                  u'about an artifact.',
                  u'filter_ops': [u'eq', u'neq', u'in'],
                  u'glareType': u'StringDict',
                  u'maxProperties': 255,
                  u'required_on_activate': False,
                  u'type': [u'object',
                            u'null']},
    u'name': {u'description': u'Artifact Name.',
              u'filter_ops': [u'eq',
                              u'neq',
                              u'in'],
              u'glareType': u'String',
              u'maxLength': 255,
              u'minLength': 1,
              u'required_on_activate': False,
              u'sortable': True,
              u'type': u'string'},
    u'owner': {u'description': u'ID of user/tenant who uploaded artifact.',
               u'filter_ops': [u'eq',
                               u'neq',
                               u'in'],
               u'glareType': u'String',
               u'maxLength': 255,
               u'readOnly': True,
               u'required_on_activate': False,
               u'sortable': True,
               u'type': u'string'},
    u'status': {u'default': u'drafted',
                u'description': u'Artifact status.',
                u'enum': [u'drafted',
                          u'active',
                          u'deactivated',
                          u'deleted'],
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'mutable': True,
                u'sortable': True,
                u'type': u'string'},
    u'tags': {u'default': [],
              u'description': u'List of tags added to Artifact.',
              u'filter_ops': [],
              u'glareType': u'StringList',
              u'items': {u'maxLength': 255,
                         u'minLength': 1,
                         u'pattern': u'^[^,/]+$',
                         u'type': u'string'},
              u'maxItems': 255,
              u'mutable': True,
              u'required_on_activate': False,
              u'type': [u'array', u'null'],
              u'uniqueItems': True},
    u'updated_at': {
        u'description': u'Datetime when artifact has been updated last time.',
        u'filter_ops': [u'lt', u'gt'],
        u'format': u'date-time',
        u'glareType': u'DateTime',
        u'mutable': True,
        u'readOnly': True,
        u'sortable': True,
        u'type': u'string'},
    u'version': {u'default': u'0.0.0',
                 u'description': u'Artifact version(semver).',
                 u'filter_ops': [u'eq',
                                 u'neq',
                                 u'in',
                                 u'gt',
                                 u'gte',
                                 u'lt',
                                 u'lte'],
                 u'glareType': u'String',
                 u'pattern': u'/^([0-9]+)\\.([0-9]+)\\.([0-9]+)(?:-'
                             u'([0-9A-Za-z-]+(?:\\.[0-9A-Za-z-]+)*))?'
                             u'(?:\\+[0-9A-Za-z-]+)?$/',
                 u'required_on_activate': False,
                 u'sortable': True,
                 u'type': u'string'},
    u'visibility': {u'default': u'private',
                    u'description': u'Artifact visibility that defines if '
                                    u'artifact can be available to other '
                                    u'users.',
                    u'enum': [u'private', u'public'],
                    u'filter_ops': [u'eq', u'neq', u'in'],
                    u'glareType': u'String',
                    u'mutable': True,
                    u'sortable': True,
                    u'type': u'string'}
}


def generate_type_props(props):
    props.update(fixture_base_props)
    return props


fixtures = {
    u'sample_artifact': {
        u'name': u'sample_artifact',
        u'properties': generate_type_props({
            u'blob': {u'additionalProperties': False,
                      u'description': u'I am Blob',
                      u'filter_ops': [],
                      u'glareType': u'Blob',
                      u'mutable': True,
                      u'properties': {
                          u'md5': {u'type': [u'string', u'null']},
                          u'sha1': {u'type': [u'string', u'null']},
                          u'sha256': {u'type': [u'string', u'null']},
                          u'content_type': {
                              u'type': u'string'},
                          u'external': {
                              u'type': u'boolean'},
                          u'size': {u'type': [
                              u'number',
                              u'null']},
                          u'status': {
                              u'enum': [
                                  u'saving',
                                  u'active'],
                              u'type': u'string'}},
                      u'required': [u'size',
                                    u'md5', u'sha1', u'sha256',
                                    u'external',
                                    u'status',
                                    u'content_type'],
                      u'required_on_activate': False,
                      u'type': [u'object',
                                u'null']},
            u'bool1': {u'default': False,
                       u'filter_ops': [u'eq'],
                       u'glareType': u'Boolean',
                       u'required_on_activate': False,
                       u'type': [u'boolean',
                                 u'null']},
            u'bool2': {u'default': False,
                       u'filter_ops': [u'eq'],
                       u'glareType': u'Boolean',
                       u'required_on_activate': False,
                       u'type': [u'boolean',
                                 u'null']},
            u'link1': {u'filter_ops': [u'eq',
                                       u'neq'],
                       u'glareType': u'Link',
                       u'required_on_activate': False,
                       u'type': [u'string',
                                 u'null']},
            u'link2': {u'filter_ops': [u'eq',
                                       u'neq'],
                       u'glareType': u'Link',
                       u'required_on_activate': False,
                       u'type': [u'string',
                                 u'null']},
            u'dict_of_blobs': {
                u'additionalProperties': {
                    u'additionalProperties': False,
                    u'properties': {
                        u'md5': {u'type': [u'string', u'null']},
                        u'sha1': {u'type': [u'string', u'null']},
                        u'sha256': {u'type': [u'string', u'null']},
                        u'content_type': {
                            u'type': u'string'},
                        u'external': {
                            u'type': u'boolean'},
                        u'size': {
                            u'type': [
                                u'number',
                                u'null']},
                        u'status': {
                            u'enum': [
                                u'saving',
                                u'active'],
                            u'type': u'string'}},
                    u'required': [u'size',
                                  u'md5', u'sha1', u'sha256',
                                  u'external',
                                  u'status',
                                  u'content_type'],
                    u'type': [u'object',
                              u'null']},
                u'default': {},
                u'filter_ops': [],
                u'glareType': u'BlobDict',
                u'maxProperties': 255,
                u'required_on_activate': False,
                u'type': [u'object',
                          u'null']},
            u'dict_of_int': {
                u'additionalProperties': {
                    u'type': u'integer'},
                u'default': {},
                u'filter_ops': [u'eq', u'in'],
                u'glareType': u'IntegerDict',
                u'maxProperties': 255,
                u'required_on_activate': False,
                u'type': [u'object',
                          u'null']},
            u'dict_of_links': {u'additionalProperties': {u'type': u'string'},
                               u'default': {},
                               u'filter_ops': [u'eq'],
                               u'glareType': u'LinkDict',
                               u'maxProperties': 255,
                               u'mutable': True,
                               u'required_on_activate': False,
                               u'type': [u'object', u'null']},
            u'list_of_links': {u'default': [],
                               u'filter_ops': [u'eq'],
                               u'glareType': u'LinkList',
                               u'items': {u'type': u'string'},
                               u'maxItems': 255,
                               u'mutable': True,
                               u'required_on_activate': False,
                               u'type': [u'array', u'null']},
            u'dict_of_str': {
                u'additionalProperties': {u'maxLength': 255,
                                          u'type': u'string'},
                u'default': {},
                u'filter_ops': [u'eq', u'in'],
                u'glareType': u'StringDict',
                u'maxProperties': 255,
                u'required_on_activate': False,
                u'type': [u'object',
                          u'null']},
            u'dict_validators': {
                u'additionalProperties': False,
                u'filter_ops': [],
                u'glareType': u'StringDict',
                u'maxProperties': 3,
                u'properties': {u'abc': {u'maxLength': 255,
                                         u'type': [u'string',
                                                   u'null']},
                                u'def': {u'maxLength': 255,
                                         u'type': [u'string',
                                                   u'null']},
                                u'ghi': {u'maxLength': 255,
                                         u'type': [u'string',
                                                   u'null']},
                                u'jkl': {u'maxLength': 255,
                                         u'type': [u'string',
                                                   u'null']}},
                u'required_on_activate': False,
                u'type': [u'object',
                          u'null']},
            u'float1': {u'filter_ops': [u'eq',
                                        u'neq',
                                        u'in',
                                        u'gt',
                                        u'gte',
                                        u'lt',
                                        u'lte'],
                        u'glareType': u'Float',
                        u'required_on_activate': False,
                        u'sortable': True,
                        u'type': [u'number',
                                  u'null']},
            u'float2': {u'filter_ops': [u'eq',
                                        u'neq',
                                        u'in',
                                        u'gt',
                                        u'gte',
                                        u'lt',
                                        u'lte'],
                        u'glareType': u'Float',
                        u'required_on_activate': False,
                        u'sortable': True,
                        u'type': [u'number',
                                  u'null']},
            u'int1': {u'filter_ops': [u'eq',
                                      u'neq',
                                      u'in',
                                      u'gt',
                                      u'gte',
                                      u'lt',
                                      u'lte'],
                      u'glareType': u'Integer',
                      u'required_on_activate': False,
                      u'sortable': True,
                      u'type': [u'integer',
                                u'null']},
            u'int2': {u'filter_ops': [u'eq',
                                      u'neq',
                                      u'in',
                                      u'gt',
                                      u'gte',
                                      u'lt',
                                      u'lte'],
                      u'glareType': u'Integer',
                      u'required_on_activate': False,
                      u'sortable': True,
                      u'type': [u'integer',
                                u'null']},
            u'int_validators': {u'filter_ops': [u'eq',
                                                u'neq',
                                                u'in',
                                                u'gt',
                                                u'gte',
                                                u'lt',
                                                u'lte'],
                                u'glareType': u'Integer',
                                u'maximum': 20,
                                u'minimum': 10,
                                u'required_on_activate': False,
                                u'type': [u'integer',
                                          u'null']},
            u'list_of_int': {u'default': [],
                             u'filter_ops': [u'eq', u'in'],
                             u'glareType': u'IntegerList',
                             u'items': {
                                 u'type': u'integer'},
                             u'maxItems': 255,
                             u'required_on_activate': False,
                             u'type': [u'array',
                                       u'null']},
            u'list_of_str': {u'default': [],
                             u'filter_ops': [u'eq', u'in'],
                             u'glareType': u'StringList',
                             u'items': {u'maxLength': 255,
                                        u'type': u'string'},
                             u'maxItems': 255,
                             u'required_on_activate': False,
                             u'type': [u'array',
                                       u'null']},
            u'list_validators': {u'default': [],
                                 u'filter_ops': [],
                                 u'glareType': u'StringList',
                                 u'items': {u'maxLength': 255,
                                            u'type': u'string'},
                                 u'maxItems': 3,
                                 u'required_on_activate': False,
                                 u'type': [u'array',
                                           u'null'],
                                 u'uniqueItems': True},
            u'small_blob': {u'additionalProperties': False,
                            u'filter_ops': [],
                            u'glareType': u'Blob',
                            u'mutable': True,
                            u'properties': {
                                u'md5': {u'type': [u'string', u'null']},
                                u'sha1': {u'type': [u'string', u'null']},
                                u'sha256': {u'type': [u'string', u'null']},
                                u'content_type': {
                                    u'type': u'string'},
                                u'external': {
                                    u'type': u'boolean'},
                                u'size': {
                                    u'type': [
                                        u'number',
                                        u'null']},
                                u'status': {
                                    u'enum': [
                                        u'saving',
                                        u'active'],
                                    u'type': u'string'}},
                            u'required': [u'size',
                                          u'md5', u'sha1', u'sha256',
                                          u'external',
                                          u'status',
                                          u'content_type'],
                            u'required_on_activate': False,
                            u'type': [u'object',
                                      u'null']},
            u'str1': {u'filter_ops': [u'eq',
                                      u'neq',
                                      u'in'],
                      u'glareType': u'String',
                      u'maxLength': 255,
                      u'required_on_activate': False,
                      u'sortable': True,
                      u'type': [u'string',
                                u'null']},
            u'string_mutable': {u'filter_ops': [u'eq',
                                                u'neq',
                                                u'in'],
                                u'glareType': u'String',
                                u'maxLength': 255,
                                u'mutable': True,
                                u'required_on_activate': False,
                                u'type': [u'string',
                                          u'null']},
            u'string_regex': {u'filter_ops': [u'eq', u'neq', u'in'],
                              u'glareType': u'String',
                              u'maxLength': 255,
                              u'pattern': u'^([0-9a-fA-F]){8}$',
                              u'required_on_activate': False,
                              u'type': [u'string', u'null']},
            u'string_required': {
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'type': [u'string',
                          u'null']},
            u'string_validators': {
                u'enum': [u'aa',
                          u'bb',
                          u'ccccccccccc'],
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 10,
                u'required_on_activate': False,
                u'type': [u'string',
                          u'null']},
            u'system_attribute': {u'default': u'default',
                                  u'filter_ops': [u'eq',
                                                  u'neq',
                                                  u'in'],
                                  u'glareType': u'String',
                                  u'maxLength': 255,
                                  u'readOnly': True,
                                  u'sortable': True,
                                  u'type': [u'string',
                                            u'null']}
        }),
        u'required': [u'name'],
        u'title': u'Artifact type sample_artifact of version 1.0',
        u'version': u'1.0',
        u'type': u'object'},
    u'tosca_templates': {
        u'name': u'tosca_templates',
        u'properties': generate_type_props({
            u'template': {
                u'additionalProperties': False,
                u'description': u'TOSCA template body.',
                u'filter_ops': [],
                u'glareType': u'Blob',
                u'properties': {
                    u'md5': {u'type': [u'string', u'null']},
                    u'sha1': {u'type': [u'string', u'null']},
                    u'sha256': {u'type': [u'string', u'null']},
                    u'content_type': {
                        u'type': u'string'},
                    u'external': {u'type': u'boolean'},
                    u'size': {u'type': [u'number',
                                        u'null']},
                    u'status': {u'enum': [u'saving',
                                          u'active'],
                                u'type': u'string'}},
                u'required': [u'size',
                              u'md5', u'sha1', u'sha256',
                              u'external',
                              u'status',
                              u'content_type'],
                u'type': [u'object',
                          u'null']},
            u'template_format': {u'description': u'TOSCA template format.',
                                 u'filter_ops': [u'eq',
                                                 u'neq',
                                                 u'in'],
                                 u'glareType': u'String',
                                 u'maxLength': 255,
                                 u'type': [u'string',
                                           u'null']},
        }),
        u'required': [u'name'],
        u'version': u'1.0',
        u'title': u'Artifact type tosca_templates of version 1.0',
        u'type': u'object'},
    u'murano_packages': {
        u'name': u'murano_packages',
        u'properties': generate_type_props({
            u'categories': {
                u'default': [],
                u'description': u'List of categories specified for '
                                u'the package.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'StringList',
                u'items': {u'maxLength': 255,
                           u'type': u'string'},
                u'maxItems': 255,
                u'mutable': True,
                u'type': [u'array',
                          u'null']},
            u'class_definitions': {
                u'default': [],
                u'description': u'List of class definitions '
                                u'in the package.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'StringList',
                u'items': {u'maxLength': 255,
                           u'type': u'string'},
                u'maxItems': 255,
                u'type': [u'array',
                          u'null'],
                u'uniqueItems': True},
            u'dependencies': {
                u'default': [],
                u'description': u'List of package dependencies for '
                                u'this package.',
                u'filter_ops': [u'eq',
                                u'neq'],
                u'glareType': u'LinkList',
                u'items': {u'type': u'string'},
                u'maxItems': 255,
                u'required_on_activate': False,
                u'type': [u'array',
                          u'null']},
            u'display_name': {
                u'description': u'Package name in human-readable format.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'mutable': True,
                u'type': [u'string',
                          u'null']},
            u'inherits': {
                u'additionalProperties': {u'maxLength': 255,
                                          u'type': u'string'},
                u'default': {},
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'StringDict',
                u'maxProperties': 255,
                u'type': [u'object',
                          u'null']},
            u'keywords': {u'default': [],
                          u'filter_ops': [u'eq',
                                          u'neq',
                                          u'in'],
                          u'glareType': u'StringList',
                          u'items': {u'maxLength': 255,
                                     u'type': u'string'},
                          u'maxItems': 255,
                          u'mutable': True,
                          u'type': [u'array',
                                    u'null']},
            u'package': {
                u'additionalProperties': False,
                u'description': u'Murano Package binary.',
                u'filter_ops': [],
                u'glareType': u'Blob',
                u'properties': {u'md5': {u'type': [u'string', u'null']},
                                u'sha1': {u'type': [u'string', u'null']},
                                u'sha256': {u'type': [u'string', u'null']},
                                u'content_type': {u'type': u'string'},
                                u'external': {u'type': u'boolean'},
                                u'size': {u'type': [u'number',
                                                    u'null']},
                                u'status': {u'enum': [u'saving',
                                                      u'active'],
                                            u'type': u'string'}},
                u'required': [u'size',
                              u'md5', u'sha1', u'sha256',
                              u'external',
                              u'status',
                              u'content_type'],
                u'required_on_activate': False,
                u'type': [u'object',
                          u'null']},
            u'type': {
                u'default': u'Application',
                u'description': u'Package type.',
                u'enum': [u'Application',
                          u'Library',
                          ],
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'type': [u'string',
                          u'null']}
        }),
        u'required': [u'name'],
        u'version': u'1.0',
        u'title': u'Artifact type murano_packages of version 1.0',
        u'type': u'object'},
    u'images': {
        u'name': u'images',
        u'properties': generate_type_props({
            u'architecture': {
                u'description': u'Operating system architecture as specified '
                                u'in http://docs.openstack.org/trunk/'
                                u'openstack-compute/admin/content/adding-'
                                u'images.html',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'required_on_activate': False,
                u'type': [u'string',
                          u'null']},
            u'container_format': {u'description': u'Image container format.',
                                  u'enum': [u'ami',
                                            u'ari',
                                            u'aki',
                                            u'bare',
                                            u'ovf',
                                            u'ova',
                                            u'docker',
                                            ],
                                  u'filter_ops': [u'eq',
                                                  u'neq',
                                                  u'in'],
                                  u'glareType': u'String',
                                  u'type': [u'string',
                                            u'null']},

            u'disk_format': {u'description': u'Image disk format.',
                             u'enum': [u'ami',
                                       u'ari',
                                       u'aki',
                                       u'vhd',
                                       u'vhdx',
                                       u'vmdk',
                                       u'raw',
                                       u'qcow2',
                                       u'vdi',
                                       u'iso',
                                       ],
                             u'filter_ops': [u'eq',
                                             u'neq',
                                             u'in'],
                             u'glareType': u'String',
                             u'type': [u'string', u'null']},
            u'image': {u'additionalProperties': False,
                       u'description': u'Image binary.',
                       u'filter_ops': [],
                       u'glareType': u'Blob',
                       u'properties': {
                           u'md5': {u'type': [u'string', u'null']},
                           u'sha1': {u'type': [u'string', u'null']},
                           u'sha256': {u'type': [u'string', u'null']},
                           u'content_type': {u'type': u'string'},
                           u'external': {u'type': u'boolean'},
                           u'size': {u'type': [u'number',
                                               u'null']},
                           u'status': {u'enum': [u'saving',
                                                 u'active'],
                                       u'type': u'string'}},
                       u'required': [u'size',
                                     u'md5', u'sha1', u'sha256',
                                     u'external',
                                     u'status',
                                     u'content_type'],
                       u'required_on_activate': False,
                       u'type': [u'object', u'null']},
            u'instance_uuid': {
                u'description': u'Metadata which can be used to record which '
                                u'instance this image is associated with. '
                                u'(Informational only, does not create an '
                                u'instance snapshot.)',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'required_on_activate': False,
                u'type': [u'string',
                          u'null']},
            u'kernel_id': {
                u'description': u'ID of image stored in Glare that should be '
                                u'used as the kernel when booting an '
                                u'AMI-style image.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'pattern': u'^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-'
                            u'([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-'
                            u'([0-9a-fA-F]){12}$',
                u'required_on_activate': False,
                u'type': [u'string', u'null']},
            u'min_disk': {
                u'description': u'Minimal disk space required to boot image.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in',
                                u'gt',
                                u'gte',
                                u'lt',
                                u'lte'],
                u'glareType': u'Integer',
                u'minimum': 0,
                u'required_on_activate': False,
                u'type': [u'integer', u'null']},
            u'min_ram': {
                u'description': u'Minimal RAM required to boot image.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in',
                                u'gt',
                                u'gte',
                                u'lt',
                                u'lte'],
                u'glareType': u'Integer',
                u'minimum': 0,
                u'required_on_activate': False,
                u'type': [u'integer', u'null']},
            u'os_distro': {
                u'description': u'Common name of operating system distribution'
                                u' as specified in http://docs.openstack.org/'
                                u'trunk/openstack-compute/admin/content/'
                                u'adding-images.html',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'required_on_activate': False,
                u'type': [u'string', u'null']},
            u'os_version': {
                u'description': u'Operating system version as specified by the'
                                u' distributor',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'required_on_activate': False,
                u'type': [u'string', u'null']},
            u'ramdisk_id': {
                u'description': u'ID of image stored in Glare that should be '
                                u'used as the ramdisk when booting an '
                                u'AMI-style image.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'pattern': u'^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F])'
                            u'{4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$',
                u'required_on_activate': False,
                u'type': [u'string', u'null']}}),
        u'required': [u'name'],
        u'version': u'1.0',
        u'title': u'Artifact type images of version 1.0',
        u'type': u'object'},
    u'heat_templates': {
        u'name': u'heat_templates',
        u'properties': generate_type_props({
            u'default_envs': {
                u'additionalProperties': {u'maxLength': 255,
                                          u'type': u'string'},
                u'default': {},
                u'description': u'Default environments that can '
                                u'be applied to the template if no '
                                u'environments specified by user.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'StringDict',
                u'maxProperties': 255,
                u'mutable': True,
                u'type': [u'object',
                          u'null']},
            u'environments': {
                u'additionalProperties': {u'type': u'string'},
                u'default': {},
                u'description': u'References to Heat Environments '
                                u'that can be used with current '
                                u'template.',
                u'filter_ops': [u'eq',
                                u'neq'],
                u'glareType': u'LinkDict',
                u'maxProperties': 255,
                u'mutable': True,
                u'type': [u'object',
                          u'null']},
            u'nested_templates': {
                u'additionalProperties':
                    {u'additionalProperties': False,
                     u'properties': {
                         u'md5': {u'type': [u'string', u'null']},
                         u'sha1': {u'type': [u'string', u'null']},
                         u'sha256': {u'type': [u'string', u'null']},
                         u'content_type': {
                             u'type': u'string'},
                         u'external': {u'type': u'boolean'},
                         u'size': {u'type': [u'number',
                                             u'null']},
                         u'status': {u'enum': [u'saving',
                                               u'active'],
                                     u'type': u'string'}},
                     u'required': [u'size',
                                   u'md5', u'sha1', u'sha256',
                                   u'external',
                                   u'status',
                                   u'content_type'],
                     u'type': [u'object',
                               u'null']},
                u'default': {},
                u'description': u'Dict of nested templates where key is the '
                                u'name  of template and value is nested '
                                u'template body.',
                u'filter_ops': [],
                u'glareType': u'BlobDict',
                u'maxProperties': 255,
                u'type': [u'object',
                          u'null']},
            u'template': {
                u'additionalProperties': False,
                u'description': u'Heat template body.',
                u'filter_ops': [],
                u'glareType': u'Blob',
                u'properties': {
                    u'md5': {u'type': [u'string', u'null']},
                    u'sha1': {u'type': [u'string', u'null']},
                    u'sha256': {u'type': [u'string', u'null']},
                    u'content_type': {
                        u'type': u'string'},
                    u'external': {u'type': u'boolean'},
                    u'size': {u'type': [u'number',
                                        u'null']},
                    u'status': {u'enum': [u'saving',
                                          u'active'],
                                u'type': u'string'}},
                u'required': [u'size',
                              u'md5', u'sha1', u'sha256',
                              u'external',
                              u'status',
                              u'content_type'],
                u'type': [u'object',
                          u'null']},

        }),
        u'version': u'1.0',
        u'required': [u'name'],
        u'title': u'Artifact type heat_templates of version 1.0',
        u'type': u'object'},
    u'heat_environments': {
        u'name': u'heat_environments',
        u'properties': generate_type_props({
            u'environment': {
                u'additionalProperties': False,
                u'description': u'Heat Environment text body.',
                u'filter_ops': [],
                u'glareType': u'Blob',
                u'properties': {u'md5': {u'type': [u'string', u'null']},
                                u'sha1': {u'type': [u'string', u'null']},
                                u'sha256': {u'type': [u'string', u'null']},
                                u'content_type': {u'type': u'string'},
                                u'external': {u'type': u'boolean'},
                                u'size': {u'type': [u'number',
                                                    u'null']},
                                u'status': {u'enum': [u'saving',
                                                      u'active'],
                                            u'type': u'string'}},
                u'required': [u'size',
                              u'md5', u'sha1', u'sha256',
                              u'external',
                              u'status',
                              u'content_type'],
                u'type': [u'object',
                          u'null']},

        }),
        u'required': [u'name'],
        u'version': u'1.0',
        u'title': u'Artifact type heat_environments of version 1.0',
        u'type': u'object'},
    u'all': {
        u'name': u'all',
        u'properties': generate_type_props({
            u'type_name': {u'description': u'Name of artifact type.',
                           u'filter_ops': [u'eq', u'neq', u'in'],
                           u'glareType': u'String',
                           u'maxLength': 255,
                           u'type': [u'string', u'null']},

        }),
        u'required': [u'name'],
        u'version': u'1.0',
        u'title': u'Artifact type all of version 1.0',
        u'type': u'object'}
}


class TestSchemas(base.TestArtifact):
    def test_schemas(self):
        # Get schemas for specific artifact type
        for at in self.enabled_types:
            result = self.get(url='/schemas/%s' % at)
            self.assertEqual(fixtures[at], result['schemas'][at])

        # Get list schemas of artifacts
        result = self.get(url='/schemas')
        self.assertEqual(fixtures, result['schemas'])

        # Validation of schemas
        result = self.get(url='/schemas')['schemas']
        for artifact_type, schema in result.items():
            jsonschema.Draft4Validator.check_schema(schema)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures

from glare.common import exception


class BannedDBSchemaOperations(fixtures.Fixture):
    """Ban some operations for migrations"""
    def __init__(self, banned_resources=None):
        super(BannedDBSchemaOperations, self).__init__()
        self._banned_resources = banned_resources or []

    @staticmethod
    def _explode(resource, op):
        raise exception.DBNotAllowed(
            'Operation %s.%s() is not allowed in a database migration' % (
                resource, op))

    def setUp(self):
        super(BannedDBSchemaOperations, self).setUp()
        for thing in self._banned_resources:
            self.useFixture(fixtures.MonkeyPatch(
                'sqlalchemy.%s.drop' % thing,
                lambda *a, **k: self._explode(thing, 'drop')))
            self.useFixture(fixtures.MonkeyPatch(
                'sqlalchemy.%s.alter' % thing,
                lambda *a, **k: self._explode(thing, 'alter')))



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import tempfile

from six import BytesIO

from glare.tests.unit import base


class TestArtifactHooks(base.BaseTestArtifactAPI):

    def test_create_hook(self):
        values = {'name': 'ttt', 'version': '1.0', 'temp_dir': self.test_dir}
        art = self.controller.create(self.req, 'hooks_artifact', values)
        self.assertEqual(self.test_dir, art['temp_dir'])
        self.assertIsNotNone(art['temp_file_path_create'])
        with open(art['temp_file_path_create']) as f:
            self.assertEqual('pre_create_hook was called\n', f.readline())
            self.assertEqual('post_create_hook was called\n', f.readline())

    def test_update_ops_hook(self):
        self.req = self.get_fake_request(user=self.users['admin'])
        values = {'name': 'ttt', 'version': '1.0', 'temp_dir': self.test_dir}
        art = self.controller.create(self.req, 'hooks_artifact', values)
        self.assertEqual(self.test_dir, art['temp_dir'])

        changes = [{'op': 'replace', 'path': '/description',
                    'value': 'some_string'},
                   {'op': 'replace', 'path': '/status',
                    'value': 'active'},
                   {'op': 'replace', 'path': '/status',
                    'value': 'deactivated'},
                   {'op': 'replace', 'path': '/status',
                    'value': 'active'},
                   {'op': 'replace', 'path': '/visibility',
                    'value': 'public'}]
        art = self.update_with_values(changes, art_type='hooks_artifact',
                                      art_id=art['id'])
        self.assertEqual('active', art['status'])
        self.assertEqual('some_string', art['description'])
        self.assertEqual('public', art['visibility'])

        actions = ['update', 'activate', 'deactivate', 'reactivate', 'publish']
        for action in actions:
            with open(art['temp_file_path_%s' % action]) as f:
                self.assertEqual('pre_%s_hook was called\n' % action,
                                 f.readline())
                self.assertEqual('post_%s_hook was called\n' % action,
                                 f.readline())

    def test_upload_download_hooks(self):
        temp_file_path = tempfile.mktemp(dir=self.test_dir)
        self.config(temp_file_path=temp_file_path,
                    group='artifact_type:hooks_artifact')

        values = {'name': 'ttt', 'version': '1.0', 'temp_dir': self.test_dir}
        art = self.controller.create(self.req, 'hooks_artifact', values)

        art = self.controller.upload_blob(
            self.req, 'hooks_artifact', art['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')
        self.assertEqual(3, art['blob']['size'])
        self.assertEqual('active', art['blob']['status'])

        self.controller.download_blob(
            self.req, 'hooks_artifact', art['id'], 'blob')

        with open(temp_file_path) as f:
            self.assertEqual('pre_upload_hook was called\n', f.readline())
            self.assertEqual('post_upload_hook was called\n', f.readline())
            self.assertEqual('pre_download_hook was called\n', f.readline())
            self.assertEqual('post_download_hook was called\n', f.readline())

    def test_add_location_hook(self):

        temp_file_path = tempfile.mktemp(dir=self.test_dir)
        self.config(temp_file_path=temp_file_path,
                    group='artifact_type:hooks_artifact')

        values = {'name': 'ttt', 'version': '1.0', 'temp_dir': self.test_dir}
        art = self.controller.create(self.req, 'hooks_artifact', values)

        ct = 'application/vnd+openstack.glare-custom-location+json'

        body = {'url': 'https://FAKE_LOCATION.com',
                'md5': "fake", 'sha1': "fake_sha", "sha256": "fake_sha256"}
        art = self.controller.upload_blob(
            self.req, 'hooks_artifact', art['id'], 'blob', body, ct)
        self.assertIsNone(art['blob']['size'])
        self.assertEqual('active', art['blob']['status'])

        # hook isn't called if we download external location
        self.controller.download_blob(
            self.req, 'hooks_artifact', art['id'], 'blob')

        with open(temp_file_path) as f:
            self.assertEqual(
                'pre_add_location_hook was called\n', f.readline())
            self.assertEqual(
                'post_add_location_hook was called\n', f.readline())

    def test_delete_hook(self):
        temp_file_path = tempfile.mktemp(dir=self.test_dir)
        self.config(temp_file_path=temp_file_path,
                    group='artifact_type:hooks_artifact')

        values = {'name': 'ttt', 'version': '1.0', 'temp_dir': self.test_dir}
        art = self.controller.create(self.req, 'hooks_artifact', values)

        self.controller.delete(self.req, 'hooks_artifact', art['id'])

        with open(temp_file_path) as f:
            self.assertEqual('pre_delete_hook was called\n', f.readline())
            self.assertEqual('post_delete_hook was called\n', f.readline())



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc
import re
import uuid

from oslo_log import log as logging
from oslo_versionedobjects import fields
import six

from glare.common import exception
from glare.i18n import _
from glare.objects.meta import fields as glare_fields

LOG = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class Validator(object):
    """Common interface for all validators."""

    @staticmethod
    @abc.abstractmethod
    def get_allowed_types():
        raise NotImplementedError()

    def check_type_allowed(self, field_type):
        if not issubclass(field_type, self.get_allowed_types()):
            # try to check if field_type is correct
            # in case of element_type passed
            allowed_field_types = tuple(type(field.AUTO_TYPE)
                                        for field in self.get_allowed_types()
                                        if hasattr(field, 'AUTO_TYPE'))
            if not issubclass(field_type, allowed_field_types):
                raise exception.IncorrectArtifactType(
                    _("%(type)s is not allowed for validator "
                      "%(val)s. Allowed types are %(allowed)s.") % {
                        "type": str(field_type),
                        "val": str(self.__class__),
                        "allowed": str(self.get_allowed_types())})

    def to_jsonschema(self):
        return {}

    @abc.abstractmethod
    def __call__(self, value):
        raise NotImplemented


class UUID(Validator):

    @staticmethod
    def get_allowed_types():
        return fields.StringField,

    def __call__(self, value):
        uuid.UUID(value, version=4)

    def to_jsonschema(self):
        return {'pattern': ('^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F])'
                            '{4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$')}


class AllowedValues(Validator):

    def __init__(self, allowed_values):
        self.allowed_values = allowed_values

    @staticmethod
    def get_allowed_types():
        return fields.StringField, fields.IntegerField, fields.FloatField

    def __call__(self, value):
        if value not in self.allowed_values:
            raise ValueError(_("Value must be one of the following: %s") %
                             ', '.join(map(str, self.allowed_values)))

    def to_jsonschema(self):
        return {'enum': self.allowed_values}


class Version(Validator):

    @staticmethod
    def get_allowed_types():
        return glare_fields.VersionField,

    def __call__(self, value):
        pass

    def to_jsonschema(self):
        return {'pattern': ('/^([0-9]+)\.([0-9]+)\.([0-9]+)(?:-([0-9A-Za-z-]'
                            '+(?:\.[0-9A-Za-z-]+)*))?(?:\+[0-9A-Za-z-]+)?$/')}


class Regex(Validator):

    def __init__(self, pattern):
        self.pattern = re.compile(pattern)

    @staticmethod
    def get_allowed_types():
        return fields.StringField,

    def __call__(self, value):
        if not self.pattern.match(value):
            raise ValueError

    def to_jsonschema(self):
        return {'pattern': self.pattern.pattern}


@six.add_metaclass(abc.ABCMeta)
class SizeValidator(Validator):

    def __init__(self, size):
        self.size = size


class MaxStrLen(SizeValidator):

    @staticmethod
    def get_allowed_types():
        return fields.StringField,

    def __call__(self, value):
        l = len(value)
        if l > self.size:
            raise ValueError(
                _("String length must be less than  %(size)d. "
                  "Current length: %(cur)d") % {'size': self.size,
                                                'cur': l})

    def to_jsonschema(self):
        return {'maxLength': self.size}


class MinStrLen(SizeValidator):

    @staticmethod
    def get_allowed_types():
        return fields.StringField,

    def __call__(self, value):
        l = len(value)
        if l < self.size:
            raise ValueError(
                _("String length must be less than  %(size)d. "
                  "Current length: %(cur)d") % {'size': self.size,
                                                'cur': l})

    def to_jsonschema(self):
        return {'minLength': self.size}


class ForbiddenChars(Validator):

    def __init__(self, forbidden_chars):
        self.forbidden_chars = forbidden_chars

    @staticmethod
    def get_allowed_types():
        return fields.StringField,

    def __call__(self, value):
        for fc in self.forbidden_chars:
            if fc in value:
                raise ValueError(
                    _("Forbidden character %(char)c found in string "
                      "%(string)s")
                    % {"char": fc, "string": value})

    def to_jsonschema(self):
        return {'pattern': '^[^%s]+$' % ''.join(self.forbidden_chars)}


@six.add_metaclass(abc.ABCMeta)
class MaxSize(SizeValidator):

    def __call__(self, value):
        l = len(value)
        if l > self.size:
            raise ValueError(
                _("Number of items must be less than  "
                  "%(size)d. Current size: %(cur)d") %
                {'size': self.size, 'cur': l})


class MaxDictSize(MaxSize):

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def to_jsonschema(self):
        return {'maxProperties': self.size}


class MaxListSize(MaxSize):

    @staticmethod
    def get_allowed_types():
        return glare_fields.List,

    def to_jsonschema(self):
        return {'maxItems': self.size}


@six.add_metaclass(abc.ABCMeta)
class MinSize(SizeValidator):

    def __call__(self, value):
        l = len(value)
        if l < self.size:
            raise ValueError(
                _("Number of items must be greater than  "
                  "%(size)d. Current size: %(cur)d") %
                {'size': self.size, 'cur': l})


class MinDictSize(MinSize):

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def to_jsonschema(self):
        return {'minProperties': self.size}


class MinListSize(MinSize):

    @staticmethod
    def get_allowed_types():
        return glare_fields.List,

    def to_jsonschema(self):
        return {'minItems': self.size}


class MaxNumberSize(SizeValidator):

    def __call__(self, value):
        if value > self.size:
            raise ValueError("Number is too big: %d. Max allowed number is "
                             "%d" % (value, self.size))

    @staticmethod
    def get_allowed_types():
        return fields.IntegerField, fields.FloatField

    def to_jsonschema(self):
        return {'maximum': self.size}


class MinNumberSize(SizeValidator):

    def __call__(self, value):
        if value < self.size:
            raise ValueError("Number is too small: %d. Min allowed number is "
                             "%d" % (value, self.size))

    @staticmethod
    def get_allowed_types():
        return fields.IntegerField, fields.FloatField

    def to_jsonschema(self):
        return {'minimum': self.size}


class Unique(Validator):

    def __init__(self, convert_to_set=False):
        self.convert_to_set = convert_to_set

    @staticmethod
    def get_allowed_types():
        return glare_fields.List,

    def __call__(self, value):
        if self.convert_to_set:
            value[:] = list(set(value))
        elif len(value) != len(set(value)):
            raise ValueError(_("List items %s must be unique.") % value)

    def to_jsonschema(self):
        return {'uniqueItems': True}


class AllowedDictKeys(Validator):

    def __init__(self, allowed_keys):
        self.allowed_items = allowed_keys

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def __call__(self, value):
        for item in value:
            if item not in self.allowed_items:
                raise ValueError(_("Key %(item)s is not allowed in dict. "
                                   "Allowed key values: %(allowed)s") %
                                 {"item": item,
                                  "allowed": ', '.join(self.allowed_items)})

    def to_jsonschema(self):
        return {
            'properties': {prop: {} for prop in self.allowed_items},
        }


class RequiredDictKeys(Validator):

    def __init__(self, required_keys):
        self.required_items = required_keys

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def __call__(self, value):
        for item in self.required_items:
            if item not in value:
                raise ValueError(_("Key \"%(item)s\" is required in "
                                   "dictionary %(value)s.") %
                                 {"item": item,
                                  "value": ''.join(
                                      '{}:{}, '.format(key, val)
                                      for key, val in value.items())})

    def to_jsonschema(self):
        return {'required': list(self.required_items)}


class MaxDictKeyLen(SizeValidator):

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def __call__(self, value):
        for key in value:
            if len(str(key)) > self.size:
                raise ValueError(_("Dict key length %(key)s must be less than "
                                   "%(size)d.") % {'key': key,
                                                   'size': self.size})


class MinDictKeyLen(SizeValidator):

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def __call__(self, value):
        for key in value:
            if len(str(key)) < self.size:
                raise ValueError(_("Dict key length %(key)s must be bigger "
                                   "than %(size)d.") % {'key': key,
                                                        'size': self.size})


@six.add_metaclass(abc.ABCMeta)
class ElementValidator(Validator):

    def __init__(self, validators):
        self.validators = validators


class ListElementValidator(ElementValidator):

    @staticmethod
    def get_allowed_types():
        return glare_fields.List,

    def __call__(self, value):
        for v in value:
            for validator in self.validators:
                validator(v)

    def to_jsonschema(self):
        return {'itemValidators': [
            val.to_jsonschema() for val in self.validators
        ]}


class DictElementValidator(ElementValidator):

    @staticmethod
    def get_allowed_types():
        return glare_fields.Dict,

    def __call__(self, value):
        for v in value.values():
            for validator in self.validators:
                validator(v)

    def to_jsonschema(self):
        return {'propertyValidators': [
            val.to_jsonschema() for val in self.validators
        ]}



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from glare_tempest_plugin import clients
from tempest.common import credentials_factory as common_creds
from tempest import config
from tempest.lib import base
from tempest.lib.common import dynamic_creds


CONF = config.CONF


class BaseArtifactTest(base.BaseTestCase):

    @classmethod
    def setUpClass(cls):
        super(BaseArtifactTest, cls).setUpClass()
        cls.resource_setup()
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    @classmethod
    def get_client_with_isolated_creds(cls, type_of_creds="admin"):
        creds = cls.get_configured_isolated_creds(
            type_of_creds=type_of_creds)

        os = clients.Manager(credentials=creds)
        client = os.artifact_client
        return client

    @classmethod
    def resource_setup(cls):
        if not CONF.service_available.glare:
            skip_msg = "Glare is disabled"
            raise cls.skipException(skip_msg)
        if not hasattr(cls, "os"):
            creds = cls.get_configured_isolated_creds(
                type_of_creds='primary')
            cls.os = clients.Manager(credentials=creds)
        cls.artifacts_client = cls.os.artifacts_client

    @classmethod
    def get_configured_isolated_creds(cls, type_of_creds='admin'):
        identity_version = CONF.identity.auth_version
        if identity_version == 'v3':
            cls.admin_role = CONF.identity.admin_role
            cls.identity_uri = CONF.identity.uri_v3
        else:
            cls.admin_role = 'admin'
            cls.identity_uri = CONF.identity.uri
        cls.dynamic_cred = dynamic_creds.DynamicCredentialProvider(
            identity_version=CONF.identity.auth_version,
            identity_uri=cls.identity_uri,
            name=cls.__name__, admin_role=cls.admin_role,
            admin_creds=common_creds.get_configured_admin_credentials(
                'identity_admin'))
        if type_of_creds == 'primary':
            creds = cls.dynamic_cred.get_primary_creds()
        elif type_of_creds == 'admin':
            creds = cls.dynamic_cred.get_admin_creds()
        elif type_of_creds == 'alt':
            creds = cls.dynamic_cred.get_alt_creds()
        else:
            creds = cls.dynamic_cred.get_credentials(type_of_creds)
        cls.dynamic_cred.type_of_creds = type_of_creds

        return creds.credentials



# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Routines for configuring Glare.
"""

import logging.config
import logging.handlers
import os

from oslo_config import cfg
from oslo_log import log as logging
from oslo_policy import policy
from paste import deploy

from glare.i18n import _

paste_deploy_opts = [
    cfg.StrOpt('flavor',
               help=_('Partial name of a pipeline in your paste configuration '
                      'file with the service name removed. For example, if '
                      'your paste section name is '
                      '[pipeline:glare-keystone] use the value '
                      '"keystone"')),
    cfg.StrOpt('config_file',
               help=_('Name of the paste configuration file.')),
]

common_opts = [
    cfg.StrOpt('digest_algorithm',
               default='sha256',
               help=_("""
Digest algorithm to use for digital signature.

Provide a string value representing the digest algorithm to
use for generating digital signatures. By default, ``sha256``
is used.

To get a list of the available algorithms supported by the version
of OpenSSL on your platform, run the command:
``openssl list-message-digest-algorithms``.
Examples are 'sha1', 'sha256', and 'sha512'.

Possible values:
    * An OpenSSL message digest algorithm identifier

Relation options:
    * None

""")),
]

CONF = cfg.CONF
CONF.register_opts(paste_deploy_opts, group='paste_deploy')
CONF.register_opts(common_opts)
policy.Enforcer(CONF)


def parse_args(args=None, usage=None, default_config_files=None):
    CONF(args=args,
         project='glare',
         usage=usage,
         default_config_files=default_config_files)


def _get_deployment_flavor(flavor=None):
    """Retrieve the paste_deploy.flavor config item, formatted appropriately
    for appending to the application name.

    :param flavor: if specified, use this setting rather than the
                   paste_deploy.flavor configuration setting
    """
    if not flavor:
        flavor = CONF.paste_deploy.flavor
    return '' if not flavor else ('-' + flavor)


def _get_paste_config_path():
    paste_suffix = '-paste.ini'
    conf_suffix = '.conf'
    if CONF.config_file:
        # Assume paste config is in a paste.ini file corresponding
        # to the last config file
        path = CONF.config_file[-1].replace(conf_suffix, paste_suffix)
    else:
        path = CONF.prog + paste_suffix
    return CONF.find_file(os.path.basename(path))


def _get_deployment_config_file():
    """Retrieve the deployment_config_file config item, formatted as an
    absolute pathname.
    """
    path = CONF.paste_deploy.config_file
    if not path:
        path = _get_paste_config_path()
    if not path:
        msg = _("Unable to locate paste config file for %s.") % CONF.prog
        raise RuntimeError(msg)
    return os.path.abspath(path)


def load_paste_app(app_name, flavor=None, conf_file=None):
    """Builds and returns a WSGI app from a paste config file.

    We assume the last config file specified in the supplied ConfigOpts
    object is the paste config file, if conf_file is None.

    :param app_name: name of the application to load
    :param flavor: name of the variant of the application to load
    :param conf_file: path to the paste config file

    :raises: RuntimeError when config file cannot be located or application
            cannot be loaded from config file
    """
    # append the deployment flavor to the application name,
    # in order to identify the appropriate paste pipeline
    app_name += _get_deployment_flavor(flavor)

    if not conf_file:
        conf_file = _get_deployment_config_file()

    logger = logging.getLogger(__name__)
    try:
        logger.debug("Loading %(app_name)s from %(conf_file)s",
                     {'conf_file': conf_file, 'app_name': app_name})

        app = deploy.loadapp("config:%s" % conf_file, name=app_name)

        # Log the options used when starting if we're in debug mode...
        if CONF.debug:
            CONF.log_opt_values(logger, logging.DEBUG)

        return app
    except (LookupError, ImportError) as e:
        msg = (_("Unable to load %(app_name)s from "
                 "configuration file %(conf_file)s."
                 "\nGot: %(e)r") % {'app_name': app_name,
                                    'conf_file': conf_file,
                                    'e': e})
        logger.error(msg)
        raise RuntimeError(msg)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_config import cfg

from glare.common import exception
from glare.db.sqlalchemy import api
from glare.i18n import _

CONF = cfg.CONF


def verify_artifact_count(context, type_name):
    """Verify if user can upload data based on his quota limits.

    :param context: user context
    :param type_name: name of artifact type
    """
    global_limit = CONF.max_artifact_number
    type_limit = getattr(
        CONF, 'artifact_type:' + type_name).max_artifact_number

    # update limits if they were reassigned for project
    project_id = context.tenant
    quotas = list_quotas(project_id).get(project_id, {})
    if 'max_artifact_number' in quotas:
        global_limit = quotas['max_artifact_number']
    if 'max_artifact_number:' + type_name in quotas:
        type_limit = quotas['max_artifact_number:' + type_name]

    session = api.get_session()
    # the whole amount of created artifacts
    whole_number = api.count_artifact_number(context, session)

    if global_limit != -1 and whole_number >= global_limit:
        msg = _("Can't create artifact because of global quota "
                "limit is %(global_limit)d artifacts. "
                "You have %(whole_number)d artifact(s).") % {
            'global_limit': global_limit, 'whole_number': whole_number}
        raise exception.Forbidden(msg)

    if type_limit is not None:
        # the amount of artifacts for specific type
        type_number = api.count_artifact_number(
            context, session, type_name)

        if type_limit != -1 and type_number >= type_limit:
            msg = _("Can't create artifact because of quota limit for "
                    "artifact type '%(type_name)s' is %(type_limit)d "
                    "artifacts. You have %(type_number)d artifact(s) "
                    "of this type.") % {
                'type_name': type_name,
                'type_limit': type_limit,
                'type_number': type_number}
            raise exception.Forbidden(msg)


def verify_uploaded_data_amount(context, type_name, data_amount=None):
    """Verify if user can upload data based on his quota limits.

    :param context: user context
    :param type_name: name of artifact type
    :param data_amount: number of bytes user wants to upload. Value None means
     that user hasn't specified data amount. In this case don't raise an
     exception, but just return the amount of data he is able to upload.
    :return: number of bytes user can upload if data_amount isn't specified
    """
    global_limit = CONF.max_uploaded_data
    type_limit = getattr(CONF, 'artifact_type:' + type_name).max_uploaded_data

    # update limits if they were reassigned for project
    project_id = context.tenant
    quotas = list_quotas(project_id).get(project_id, {})
    if 'max_uploaded_data' in quotas:
        global_limit = quotas['max_uploaded_data']
    if 'max_uploaded_data:' + type_name in quotas:
        type_limit = quotas['max_uploaded_data:' + type_name]

    session = api.get_session()
    # the whole amount of created artifacts
    whole_number = api.calculate_uploaded_data(context, session)
    res = -1

    if global_limit != -1:
        if data_amount is None:
            res = global_limit - whole_number
        elif whole_number + data_amount > global_limit:
            msg = _("Can't upload %(data_amount)d byte(s) because of global "
                    "quota limit: %(global_limit)d. "
                    "You have %(whole_number)d bytes uploaded.") % {
                'data_amount': data_amount,
                'global_limit': global_limit,
                'whole_number': whole_number}
            raise exception.RequestEntityTooLarge(msg)

    if type_limit is not None:
        # the amount of artifacts for specific type
        type_number = api.calculate_uploaded_data(
            context, session, type_name)
        if type_limit != -1:
            if data_amount is None:
                available = type_limit - type_number
                res = available if res == -1 else min(res, available)
            elif type_number + data_amount > type_limit:
                msg = _("Can't upload %(data_amount)d byte(s) because of "
                        "quota limit for artifact type '%(type_name)s': "
                        "%(type_limit)d. You have %(type_number)d bytes "
                        "uploaded for this type.") % {
                    'data_amount': data_amount,
                    'type_name': type_name,
                    'type_limit': type_limit,
                    'type_number': type_number}
                raise exception.RequestEntityTooLarge(msg)
    return res


def set_quotas(values):
    session = api.get_session()
    api.set_quotas(values, session)


def list_quotas(project_id=None):
    session = api.get_session()
    return api.get_all_quotas(session, project_id)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Sample artifact object for testing purposes"""

from oslo_versionedobjects import fields

from glare.objects import base as base_artifact
from glare.objects.meta import fields as glare_fields
from glare.objects.meta import validators
from glare.objects.meta import wrappers

Field = wrappers.Field.init
Dict = wrappers.DictField.init
List = wrappers.ListField.init
Blob = wrappers.BlobField.init
Folder = wrappers.FolderField.init


class SampleArtifact(base_artifact.BaseArtifact):
    VERSION = '1.0'

    fields = {
        'blob': Blob(required_on_activate=False, mutable=True,
                     description="I am Blob"),
        'small_blob': Blob(max_blob_size=10, required_on_activate=False,
                           mutable=True),
        'link1': Field(glare_fields.Link,
                       required_on_activate=False),
        'link2': Field(glare_fields.Link,
                       required_on_activate=False),
        'bool1': Field(fields.FlexibleBooleanField,
                       required_on_activate=False,
                       filter_ops=(wrappers.FILTER_EQ,),
                       default=False),
        'bool2': Field(fields.FlexibleBooleanField,
                       required_on_activate=False,
                       filter_ops=(wrappers.FILTER_EQ,),
                       default=False),
        'int1': Field(fields.IntegerField,
                      required_on_activate=False,
                      sortable=True),
        'int2': Field(fields.IntegerField,
                      sortable=True,
                      required_on_activate=False),
        'float1': Field(fields.FloatField,
                        sortable=True,
                        required_on_activate=False),
        'float2': Field(fields.FloatField,
                        sortable=True,
                        required_on_activate=False),
        'str1': Field(fields.StringField,
                      sortable=True,
                      required_on_activate=False),
        'list_of_str': List(fields.String,
                            required_on_activate=False,
                            filter_ops=(wrappers.FILTER_EQ,
                                        wrappers.FILTER_IN)),
        'list_of_int': List(fields.Integer,
                            required_on_activate=False,
                            filter_ops=(wrappers.FILTER_EQ,
                                        wrappers.FILTER_IN)),
        'dict_of_str': Dict(fields.String,
                            required_on_activate=False,
                            filter_ops=(wrappers.FILTER_EQ,
                                        wrappers.FILTER_IN)),
        'dict_of_int': Dict(fields.Integer,
                            required_on_activate=False,
                            filter_ops=(wrappers.FILTER_EQ,
                                        wrappers.FILTER_IN)),
        'dict_of_links': Dict(glare_fields.LinkFieldType,
                              mutable=True,
                              required_on_activate=False,
                              filter_ops=(wrappers.FILTER_EQ,)),
        'list_of_links': List(glare_fields.LinkFieldType,
                              mutable=True,
                              required_on_activate=False,
                              filter_ops=(wrappers.FILTER_EQ,)),
        'dict_of_blobs': Folder(required_on_activate=False,
                                max_folder_size=2000,
                                validators=[
                                    validators.MaxDictKeyLen(1000)]),
        'string_mutable': Field(fields.StringField,
                                required_on_activate=False,
                                mutable=True),
        'string_regex': Field(fields.StringField,
                              required_on_activate=False,
                              validators=[
                                  validators.Regex('^([0-9a-fA-F]){8}$')]),
        'string_required': Field(fields.StringField,
                                 required_on_activate=True),
        'string_validators': Field(fields.StringField,
                                   required_on_activate=False,
                                   validators=[
                                       validators.AllowedValues(
                                           ['aa', 'bb', 'c' * 11]),
                                       validators.MaxStrLen(10)
                                   ]),
        'int_validators': Field(fields.IntegerField,
                                required_on_activate=False,
                                validators=[
                                    validators.MinNumberSize(10),
                                    validators.MaxNumberSize(20)
                                ]),
        'list_validators': List(fields.String,
                                required_on_activate=False,
                                filter_ops=[],
                                max_size=3,
                                validators=[validators.Unique()]),
        'dict_validators': Dict(fields.String,
                                required_on_activate=False,
                                default=None,
                                filter_ops=[],
                                validators=[
                                    validators.AllowedDictKeys([
                                        'abc', 'def', 'ghi', 'jkl'])],
                                max_size=3),
        'system_attribute': Field(fields.StringField,
                                  system=True, sortable=True,
                                  default="default")
    }

    @classmethod
    def get_type_name(cls):
        return "sample_artifact"



# Copyright (c) 2016 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json

from tempest import config
from tempest.lib.common import rest_client


CONF = config.CONF


class ArtifactsClient(rest_client.RestClient):

    def __init__(self, auth_provider):
        super(ArtifactsClient, self).__init__(
            auth_provider,
            CONF.artifacts.catalog_type,
            CONF.identity.region,
            endpoint_type=CONF.artifacts.endpoint_type)

    def create_artifact(self, type_name, name, version='0.0.0', **kwargs):
        kwargs.update({'name': name, 'version': version})
        uri = '/artifacts/{type_name}'.format(type_name=type_name)
        resp, body = self.post(uri, body=json.dumps(kwargs))
        self.expected_success(201, resp.status)
        parsed = self._parse_resp(body)
        return parsed

    def get_artifact(self, type_name, art_id):
        uri = '/artifacts/{type_name}/{id}'.format(
              type_name=type_name,
              id=art_id)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        parsed = self._parse_resp(body)
        return parsed

    def update_artifact(self, type_name, art_id, remove_props=None, **kwargs):
        headers = {'Content-Type': 'application/json-patch+json'}
        uri = '/artifacts/{type_name}/{id}'.format(type_name=type_name,
                                                   id=art_id)
        changes = []

        if remove_props:
            for prop_name in remove_props:
                if prop_name not in kwargs:
                    if '/' in prop_name:
                        changes.append({'op': 'remove',
                                        'path': '/%s' % prop_name})
                    else:
                        changes.append({'op': 'replace',
                                        'path': '/%s' % prop_name,
                                        'value': None})
        for prop_name in kwargs:
            changes.append({'op': 'add',
                            'path': '/%s' % prop_name,
                            'value': kwargs[prop_name]})
        resp, body = self.patch(uri, json.dumps(changes), headers=headers)
        self.expected_success(200, resp.status)
        parsed = self._parse_resp(body)
        return parsed

    def activate_artifact(self, type_name, art_id):
        return self.update_artifact(type_name, art_id, status='active')

    def deactivate_artifact(self, type_name, art_id):
        return self.update_artifact(type_name, art_id, status='deactivated')

    def reactivate_artifact(self, type_name, art_id):
        return self.update_artifact(type_name, art_id, status='active')

    def publish_artifact(self, type_name, art_id):
        return self.update_artifact(type_name, art_id, visibility='public')

    def upload_blob(self, type_name, art_id, blob_property, data):
        headers = {'Content-Type': 'application/octet-stream'}
        uri = '/artifacts/{type_name}/{id}/{blob_prop}'.format(
            type_name=type_name,
            id=art_id,
            blob_prop=blob_property)
        resp, body = self.put(uri, data, headers=headers)
        self.expected_success(200, resp.status)
        parsed = self._parse_resp(body)
        return parsed

    def download_blob(self, type_name, art_id, blob_property):
        uri = '/artifacts/{type_name}/{id}/{blob_prop}'.format(
            type_name=type_name,
            id=art_id,
            blob_prop=blob_property)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        parsed = self._parse_resp(body)
        return parsed

    def delete_artifact(self, type_name, art_id):
        uri = '/artifacts/{type_name}/{id}'.format(
            type_name=type_name,
            id=art_id)
        self.delete(uri)

    def list_artifacts(self, type_name):
        uri = '/artifacts/{}'.format(type_name)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        parsed = self._parse_resp(body)
        return parsed



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from glare.api.v1 import resource
from glare.common import wsgi


class API(wsgi.Router):
    """WSGI router for Glare v1 API requests.

    API Router redirects incoming requests to appropriate WSGI resource method.
    """

    def __init__(self, mapper):

        glare_resource = resource.create_resource()
        reject_method_resource = wsgi.Resource(wsgi.RejectMethodController())

        # ---schemas---
        mapper.connect('/schemas',
                       controller=glare_resource,
                       action='list_type_schemas',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/schemas',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET')

        mapper.connect('/schemas/{type_name}',
                       controller=glare_resource,
                       action='show_type_schema',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/schemas/{type_name}',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET')

        # ---artifacts---
        mapper.connect('/artifacts/{type_name}',
                       controller=glare_resource,
                       action='list',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/artifacts/{type_name}',
                       controller=glare_resource,
                       action='create',
                       conditions={'method': ['POST']})
        mapper.connect('/artifacts/{type_name}',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET, POST')

        mapper.connect('/artifacts/{type_name}/{artifact_id}',
                       controller=glare_resource,
                       action='update',
                       conditions={'method': ['PATCH']})
        mapper.connect('/artifacts/{type_name}/{artifact_id}',
                       controller=glare_resource,
                       action='show',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/artifacts/{type_name}/{artifact_id}',
                       controller=glare_resource,
                       action='delete',
                       conditions={'method': ['DELETE']},
                       body_reject=True)
        mapper.connect('/artifacts/{type_name}/{artifact_id}',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET, PATCH, DELETE')

        # ---blobs---
        mapper.connect('/artifacts/{type_name}/{artifact_id}/{blob_path:.*?}',
                       controller=glare_resource,
                       action='download_blob',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/artifacts/{type_name}/{artifact_id}/{blob_path:.*?}',
                       controller=glare_resource,
                       action='upload_blob',
                       conditions={'method': ['PUT']})
        mapper.connect('/artifacts/{type_name}/{artifact_id}/{blob_path:.*?}',
                       controller=glare_resource,
                       action='delete_external_blob',
                       conditions={'method': ['DELETE']})
        mapper.connect('/artifacts/{type_name}/{artifact_id}/{blob_path:.*?}',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET, PUT, DELETE')

        # ---quotas---
        mapper.connect('/quotas',
                       controller=glare_resource,
                       action='set_quotas',
                       conditions={'method': ['PUT']})
        mapper.connect('/quotas',
                       controller=glare_resource,
                       action='list_all_quotas',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/quotas',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='PUT, GET')

        mapper.connect('/project-quotas',
                       controller=glare_resource,
                       action='list_project_quotas',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/project-quotas',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET')
        mapper.connect('/project-quotas/{project_id}',
                       controller=glare_resource,
                       action='list_project_quotas',
                       conditions={'method': ['GET']},
                       body_reject=True)
        mapper.connect('/project-quotas/{project_id}',
                       controller=reject_method_resource,
                       action='reject',
                       allowed_methods='GET')

        super(API, self).__init__(mapper)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_db.sqlalchemy import models
from oslo_utils import timeutils
from oslo_utils import uuidutils
from sqlalchemy import BigInteger
from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy.ext import declarative
from sqlalchemy import ForeignKey
from sqlalchemy import Index
from sqlalchemy import Integer
from sqlalchemy import LargeBinary
from sqlalchemy import Numeric
from sqlalchemy.orm import backref
from sqlalchemy.orm import composite
from sqlalchemy.orm import relationship
from sqlalchemy import String
from sqlalchemy import Text

from glare.common import semver_db

BASE = declarative.declarative_base()


class ArtifactBase(models.ModelBase):
    """Base class for Artifact Models."""

    __table_args__ = {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'}
    __table_initialized__ = False

    def save(self, session=None):
        from glare.db.sqlalchemy import api as db_api

        super(ArtifactBase, self).save(session or db_api.get_session())

    def keys(self):
        return self.__dict__.keys()

    def values(self):
        return self.__dict__.values()

    def items(self):
        return self.__dict__.items()

    def to_dict(self):
        d = {}
        for c in self.__table__.columns:
            d[c.name] = self[c.name]
        return d


def _parse_property_value(prop):
    columns = [
        'int_value',
        'string_value',
        'bool_value',
        'numeric_value']

    for prop_type in columns:
        if getattr(prop, prop_type) is not None:
            return getattr(prop, prop_type)


def _parse_blob_value(blob):
    return {
        "id": blob.id,
        "url": blob.url,
        "status": blob.status,
        "external": blob.external,
        "md5": blob.md5,
        "sha1": blob.sha1,
        "sha256": blob.sha256,
        "size": blob.size,
        "content_type": blob.content_type
    }


class Artifact(BASE, ArtifactBase):
    __tablename__ = 'glare_artifacts'
    __table_args__ = (
        Index('ix_glare_artifact_name_and_version', 'name', 'version_prefix',
              'version_suffix'),
        Index('ix_glare_artifact_type', 'type_name'),
        Index('ix_glare_artifact_status', 'status'),
        Index('ix_glare_artifact_owner', 'owner'),
        Index('ix_glare_artifact_visibility', 'visibility'),
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'})
    __protected_attributes__ = set(["created_at", "updated_at"])

    id = Column(String(36), primary_key=True,
                default=lambda: uuidutils.generate_uuid())
    name = Column(String(255), nullable=False)
    type_name = Column(String(255), nullable=False)
    version_prefix = Column(BigInteger().with_variant(Integer, "sqlite"),
                            nullable=False)
    version_suffix = Column(String(255))
    version_meta = Column(String(255))
    version = composite(semver_db.DBVersion, version_prefix,
                        version_suffix, version_meta,
                        comparator_factory=semver_db.VersionComparator)
    description = Column(Text())
    visibility = Column(String(32), nullable=False)
    status = Column(String(32), nullable=False)
    owner = Column(String(255))
    created_at = Column(DateTime, default=lambda: timeutils.utcnow(),
                        nullable=False)
    updated_at = Column(DateTime, default=lambda: timeutils.utcnow(),
                        nullable=False, onupdate=lambda: timeutils.utcnow())
    activated_at = Column(DateTime)

    def to_dict(self):
        d = super(Artifact, self).to_dict()

        d.pop('version_prefix')
        d.pop('version_suffix')
        d.pop('version_meta')
        d['version'] = str(self.version)

        # parse tags
        tags = []
        for tag in self.tags:
            tags.append(tag.value)
        d['tags'] = tags

        # parse properties
        for prop in self.properties:
            prop_value = _parse_property_value(prop)

            if prop.position is not None:
                if prop.name not in d:
                    # create new list
                    d[prop.name] = []
                # insert value in position
                d[prop.name].insert(prop.position, prop_value)
            elif prop.key_name is not None:
                if prop.name not in d:
                    # create new dict
                    d[prop.name] = {}
                # insert value in the dict
                d[prop.name][prop.key_name] = prop_value
            else:
                # make scalar
                d[prop.name] = prop_value

        # parse blobs
        for blob in self.blobs:
            blob_value = _parse_blob_value(blob)
            if blob.key_name is not None:
                if blob.name not in d:
                    # create new dict
                    d[blob.name] = {}
                # insert value in the dict
                d[blob.name][blob.key_name] = blob_value
            else:
                # make scalar
                d[blob.name] = blob_value

        return d


class ArtifactTag(BASE, ArtifactBase):
    __tablename__ = 'glare_artifact_tags'
    __table_args__ = (Index('ix_glare_artifact_tags_artifact_id_tag_value',
                            'artifact_id', 'value'),
                      Index('ix_glare_artifact_tags_artifact_id',
                            'artifact_id'),
                      {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'},)

    id = Column(String(36), primary_key=True, nullable=False,
                default=lambda: uuidutils.generate_uuid())
    artifact_id = Column(String(36), ForeignKey('glare_artifacts.id'),
                         nullable=False)
    artifact = relationship(Artifact,
                            backref=backref('tags',
                                            cascade="all, delete-orphan"))
    value = Column(String(255), nullable=False)


class ArtifactProperty(BASE, ArtifactBase):
    __tablename__ = 'glare_artifact_properties'
    __table_args__ = (
        Index('ix_glare_artifact_properties_artifact_id', 'artifact_id'),
        Index('ix_glare_artifact_properties_name', 'name'),
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'},)
    id = Column(String(36), primary_key=True, nullable=False,
                default=lambda: uuidutils.generate_uuid())
    artifact_id = Column(String(36), ForeignKey('glare_artifacts.id'),
                         nullable=False)
    artifact = relationship(Artifact,
                            backref=backref('properties',
                                            cascade="all, delete-orphan"))
    name = Column(String(255), nullable=False)
    string_value = Column(String(20000))
    int_value = Column(Integer)
    numeric_value = Column(Numeric)
    bool_value = Column(Boolean)
    position = Column(Integer)
    key_name = Column(String(255))


class ArtifactBlob(BASE, ArtifactBase):
    __tablename__ = 'glare_artifact_blobs'
    __table_args__ = (
        Index('ix_glare_artifact_blobs_artifact_id', 'artifact_id'),
        Index('ix_glare_artifact_blobs_name', 'name'),
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'},)
    id = Column(String(36), primary_key=True, nullable=False,
                default=lambda: uuidutils.generate_uuid())
    artifact_id = Column(String(36), ForeignKey('glare_artifacts.id'),
                         nullable=False)
    name = Column(String(255), nullable=False)
    size = Column(BigInteger().with_variant(Integer, "sqlite"))
    md5 = Column(String(32))
    sha1 = Column(String(40))
    sha256 = Column(String(64))
    external = Column(Boolean)
    url = Column(Text)
    status = Column(String(32), nullable=False)
    key_name = Column(String(2048))
    content_type = Column(String(255))
    artifact = relationship(Artifact,
                            backref=backref('blobs',
                                            cascade="all, delete-orphan"))


class ArtifactLock(BASE, ArtifactBase):
    __tablename__ = 'glare_artifact_locks'
    __table_args__ = (
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'},)
    id = Column(String(255), primary_key=True, nullable=False)
    acquired_at = Column(
        DateTime, nullable=False, default=lambda: timeutils.utcnow())


class ArtifactBlobData(BASE, ArtifactBase):
    __tablename__ = 'glare_blob_data'
    __table_args__ = (
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'},)
    id = Column(String(255), primary_key=True, nullable=False)
    data = Column(LargeBinary(length=(2 ** 32) - 1), nullable=False)


class ArtifactQuota(BASE, ArtifactBase):
    __tablename__ = 'glare_quotas'
    __table_args__ = (
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'},)
    project_id = Column(String(255), primary_key=True)
    quota_name = Column(String(32), primary_key=True)
    quota_value = Column(BigInteger().with_variant(Integer, "sqlite"),
                         nullable=False)


def register_models(engine):
    """Create database tables for all models with the given engine."""
    models = (Artifact, ArtifactTag, ArtifactProperty, ArtifactBlob,
              ArtifactLock, ArtifactQuota)
    for model in models:
        model.metadata.create_all(engine)


def unregister_models(engine):
    """Drop database tables for all models with the given engine."""
    models = (ArtifactQuota, ArtifactLock, ArtifactBlob, ArtifactProperty,
              ArtifactTag, Artifact)
    for model in models:
        model.metadata.drop_all(engine)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from glare.tests.functional import base


class TestAll(base.TestArtifact):

    def test_all(self):
        for type_name in self.enabled_types:
            if type_name == 'all':
                continue
            for i in range(3):
                for j in range(3):
                    self.create_artifact(
                        data={'name': '%s_%d' % (type_name, i),
                              'version': '%d' % j,
                              'tags': ['tag%s' % i]},
                        type_name=type_name)

        # get all possible artifacts
        url = '/all?sort=name:asc&limit=100'
        res = self.get(url=url, status=200)['all']
        from pprint import pformat
        self.assertEqual(54, len(res), pformat(res))

        # get artifacts with latest versions
        url = '/all?version=latest&sort=name:asc'
        res = self.get(url=url, status=200)['all']
        self.assertEqual(18, len(res))
        for art in res:
            self.assertEqual('2.0.0', art['version'])

        # get images only
        url = '/all?type_name=images&sort=name:asc'
        res = self.get(url=url, status=200)['all']
        self.assertEqual(9, len(res))
        for art in res:
            self.assertEqual('images', art['type_name'])

        # get images and heat_templates
        url = '/all?type_name=in:images,heat_templates&sort=name:asc'
        res = self.get(url=url, status=200)['all']
        self.assertEqual(18, len(res))
        for art in res:
            self.assertIn(art['type_name'], ('images', 'heat_templates'))

    def test_all_readonlyness(self):
        self.create_artifact(data={'name': 'all'}, type_name='all', status=403)
        art = self.create_artifact(data={'name': 'image'}, type_name='images')

        url = '/all/%s' % art['id']

        # update 'all' is forbidden
        data = [{
            "op": "replace",
            "path": "/description",
            "value": "text"
        }]
        self.patch(url=url, data=data, status=403)

        # activation is forbidden
        data = [{
            "op": "replace",
            "path": "/status",
            "value": "active"
        }]
        self.patch(url=url, data=data, status=403)

        # publishing is forbidden
        data = [{
            "op": "replace",
            "path": "/visibility",
            "value": "public"
        }]
        self.patch(url=url, data=data, status=403)

        # get is okay
        new_art = self.get(url=url)
        self.assertEqual(new_art['id'], art['id'])



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from glare.common import exception as exc
from glare.tests.unit import base


class TestArtifactCreate(base.BaseTestArtifactAPI):

    """Test Glare artifact creation."""

    def test_create_artifact_minimal(self):

        for name in ['ttt', 'tt:t', 'tt t', 'tt: t', 'tt,t']:
            values = {'name': name}

            res = self.controller.create(self.req, 'sample_artifact', values)
            self.assertEqual(name, res['name'])
            self.assertEqual('0.0.0', res['version'])
            self.assertEqual(self.users['user1']['tenant_id'], res['owner'])
            self.assertEqual('drafted', res['status'])
            self.assertEqual('private', res['visibility'])
            self.assertEqual('', res['description'])
            self.assertEqual({}, res['metadata'])
            self.assertEqual([], res['tags'])

    def test_create_artifact_with_version(self):
        values = {'name': 'name', 'version': '1.0'}
        res = self.controller.create(self.req, 'sample_artifact', values)
        self.assertEqual('name', res['name'])
        self.assertEqual('1.0.0', res['version'])

        values = {'name': 'name', 'version': '1:0'}
        res = self.controller.create(self.req, 'sample_artifact', values)
        self.assertEqual('1.0.0-0', res['version'])

        values = {'name': 'name', 'version': '1:0:0'}
        res = self.controller.create(self.req, 'sample_artifact', values)
        self.assertEqual('1.0.0-0-0', res['version'])

        values = {'name': 'name', 'version': '2:0-0'}
        res = self.controller.create(self.req, 'sample_artifact', values)
        self.assertEqual('2.0.0-0-0', res['version'])

    def test_create_artifact_with_fields(self):
        values = {'name': 'ttt', 'version': '1.0',
                  'description': "Test Artifact", 'tags': ['a', 'a', 'b'],
                  'metadata': {'type': 'image'}}

        res = self.controller.create(self.req, 'sample_artifact', values)
        self.assertEqual('ttt', res['name'])
        self.assertEqual('1.0.0', res['version'])
        self.assertEqual(self.users['user1']['tenant_id'], res['owner'])
        self.assertEqual('drafted', res['status'])
        self.assertEqual('private', res['visibility'])
        self.assertEqual('Test Artifact', res['description'])
        self.assertEqual({'type': 'image'}, res['metadata'])
        self.assertEqual({'a', 'b'}, set(res['tags']))

    def test_create_no_artifact_type(self):
        values = {'name': 'ttt'}

        self.assertRaises(exc.NotFound, self.controller.create,
                          self.req, 'wrong_type', values)

    def test_create_artifact_no_name(self):
        values = {'version': '1.0'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

    def test_create_artifact_wrong_parameters(self):
        values = {'name': 'test', 'version': 'invalid_format'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'version': -1}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'version': ':'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': '', 'version': '1.0'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'a' * 256}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'description': 'a' * 4097}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'tags': ['a' * 256]}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'tags': ['']}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'tags': ['a/a']}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'tags': ['a,a']}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'tags': [str(i) for i in range(256)]}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'metadata': {'key': 'a' * 256}}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'metadata': {'': 'a'}}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'metadata': {'a' * 256: 'a'}}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test',
                  'metadata': {('a' + str(i)): 'a' for i in range(256)}}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

    def test_create_artifact_not_existing_field(self):
        values = {'name': 'test', 'not_exist': 'some_value'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', '': 'a'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

    def test_create_artifact_blob(self):
        values = {'name': 'test', 'blob': 'DATA'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

    def test_create_artifact_system_fields(self):
        values = {'name': 'test',
                  'id': '5fdeba9a-ba12-4147-bb8a-a8daada84222'}
        self.assertRaises(exc.Forbidden, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'created_at': '2000-01-01'}
        self.assertRaises(exc.Forbidden, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'updated_at': '2000-01-01'}
        self.assertRaises(exc.Forbidden, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'activated_at': '2000-01-01'}
        self.assertRaises(exc.Forbidden, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'owner': 'new_owner'}
        self.assertRaises(exc.Forbidden, self.controller.create,
                          self.req, 'sample_artifact', values)

    def test_create_artifact_status_and_visibility(self):
        values = {'name': 'test', 'status': 'activated'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

        values = {'name': 'test', 'visibility': 'public'}
        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'sample_artifact', values)

    def test_create_artifact_unicode(self):
        name = u'\u0442\u0435\u0441\u0442'
        description = u'\u041E\u043F\u0438\u0441\u0430\u043D\u0438\u0435'
        tags = [u'\u041C\u0435\u0442\u043A\u0430']
        metadata = {'key': u'\u0417\u043D\u0430\u0447\u0435\u043D\u0438\u0435'}
        values = {
            'name': name,
            'version': '1.0',
            'description': description,
            'tags': tags,
            'metadata': metadata
        }

        res = self.controller.create(self.req, 'images', values)
        self.assertEqual(name, res['name'])
        self.assertEqual('1.0.0', res['version'])
        self.assertEqual(self.users['user1']['tenant_id'], res['owner'])
        self.assertEqual('drafted', res['status'])
        self.assertEqual('private', res['visibility'])
        self.assertEqual(description, res['description'])
        self.assertEqual(metadata, res['metadata'])
        self.assertEqual(tags, res['tags'])

    def test_create_artifact_4_byte_unicode(self):
        bad_name = u'A name with forbidden symbol \U0001f62a'
        values = {
            'name': bad_name,
            'version': '1.0',
        }

        self.assertRaises(exc.BadRequest, self.controller.create,
                          self.req, 'images', values)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects import fields

from glare.objects import base
from glare.objects.meta import fields as glare_fields
from glare.objects.meta import wrappers

Field = wrappers.Field.init
Blob = wrappers.BlobField.init
Dict = wrappers.DictField.init
Folder = wrappers.FolderField.init


class HeatTemplate(base.BaseArtifact):

    fields = {
        'environments': Dict(glare_fields.LinkFieldType,
                             mutable=True,
                             description="References to Heat Environments "
                                         "that can be used with current "
                                         "template."),
        'template': Blob(description="Heat template body."),
        'nested_templates': Folder(description="Dict of nested templates "
                                               "where key is the name  of "
                                               "template and value is "
                                               "nested template body."),
        'default_envs': Dict(fields.String, mutable=True,
                             description="Default environments that can be "
                                         "applied to the template if no "
                                         "environments specified by user.")
    }

    @classmethod
    def get_type_name(cls):
        return "heat_templates"



# Copyright 2016 Openstack Foundation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from glare.common import exception
from glare.i18n import _


REST_API_VERSION_HISTORY = """REST API Version History:

    * 1.1 Added dynamic quotas API request. Added a possibility to delete blobs
    with external locations. Added a possibility to define system locations to
    blobs.

    * 1.0 - First stable API version that supports microversion. If API version
    is not specified in the request then API v1.0 is used as default API
    version.
"""


class APIVersionRequest(object):
    """This class represents an API Version Request with convenience
    methods for manipulation and comparison of version
    numbers that we need to do to implement microversions.
    """

    _MIN_API_VERSION = "1.0"
    _MAX_API_VERSION = "1.1"
    _DEFAULT_API_VERSION = "1.0"

    def __init__(self, version_string):
        """Create an API version request object.

        :param version_string: String representation of APIVersionRequest.
         Correct format is 'X.Y', where 'X' and 'Y' are int values.
        """
        match = re.match(r"^([1-9]\d*)\.([1-9]\d*|0)$", version_string)
        if match:
            self.ver_major = int(match.group(1))
            self.ver_minor = int(match.group(2))
        else:
            msg = _("API version string %s is not valid. "
                    "Cannot determine API version.") % version_string
            raise exception.BadRequest(msg)

    def __str__(self):
        """Debug/Logging representation of object."""
        return ("API Version Request Major: %s, Minor: %s"
                % (self.ver_major, self.ver_minor))

    def _format_type_error(self, other):
        return TypeError(_("'%(other)s' should be an instance of '%(cls)s'") %
                         {"other": other, "cls": self.__class__})

    def __lt__(self, other):
        if not isinstance(other, APIVersionRequest):
            raise self._format_type_error(other)

        return ((self.ver_major, self.ver_minor) <
                (other.ver_major, other.ver_minor))

    def __eq__(self, other):
        if not isinstance(other, APIVersionRequest):
            raise self._format_type_error(other)

        return ((self.ver_major, self.ver_minor) ==
                (other.ver_major, other.ver_minor))

    def __gt__(self, other):
        if not isinstance(other, APIVersionRequest):
            raise self._format_type_error(other)

        return ((self.ver_major, self.ver_minor) >
                (other.ver_major, other.ver_minor))

    def __le__(self, other):
        return self < other or self == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __ge__(self, other):
        return self > other or self == other

    def matches(self, min_version, max_version):
        """Returns whether the version object represents a version
        greater than or equal to the minimum version and less than
        or equal to the maximum version.

        :param min_version: Minimum acceptable version.
        :param max_version: Maximum acceptable version.
        :returns: boolean
        """
        return min_version <= self <= max_version

    def get_string(self):
        """Converts object to string representation which is used to create
        an APIVersionRequest object results in the same version request.
        """
        return "%s.%s" % (self.ver_major, self.ver_minor)

    @classmethod
    def min_version(cls):
        """Minimal allowed api version."""
        return APIVersionRequest(cls._MIN_API_VERSION)

    @classmethod
    def max_version(cls):
        """Maximal allowed api version."""
        return APIVersionRequest(cls._MAX_API_VERSION)

    @classmethod
    def default_version(cls):
        """Default api version if no version in request."""
        return APIVersionRequest(cls._DEFAULT_API_VERSION)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from six import BytesIO

from glare.db.sqlalchemy import api
from glare.tests.unit import base


class TestQuotaFunctions(base.BaseTestArtifactAPI):
    """Test quota db functions."""

    def setUp(self):
        super(TestQuotaFunctions, self).setUp()
        self.session = api.get_session()

    def test_count_artifact_number(self):
        # initially there are no artifacts
        self.assertEqual(0, api.count_artifact_number(
            self.req.context, self.session))

        # create 5 images, 3 heat templates, 2 murano packages and 7 samples
        amount = {
            'images': 5,
            'heat_templates': 3,
            'murano_packages': 2,
            'sample_artifact': 7
        }
        for type_name in amount:
            for num in range(amount[type_name]):
                self.controller.create(
                    self.req, type_name, {'name': type_name + str(num)})

        # create 1 artifact of each type from different user
        req = self.get_fake_request(self.users['user2'])
        for type_name in amount:
            self.controller.create(req, type_name, {'name': type_name})

        # count numbers for each type
        for type_name in amount:
            num = api.count_artifact_number(
                self.req.context, self.session, type_name)
            self.assertEqual(amount[type_name], num)

        # count the whole amount of artifacts
        self.assertEqual(17, api.count_artifact_number(
            self.req.context, self.session))

    def test_calculate_uploaded_data(self):
        # initially there is no data
        self.assertEqual(0, api.calculate_uploaded_data(
            self.req.context, self.session))

        # create a sample artifact
        art1 = self.controller.create(
            self.req, 'sample_artifact', {'name': 'art1'})

        # upload 10 bytes to 'blob'
        self.controller.upload_blob(
            self.req, 'sample_artifact', art1['id'], 'blob',
            BytesIO(b'a' * 10), 'application/octet-stream')
        self.assertEqual(10, api.calculate_uploaded_data(
            self.req.context, self.session))

        # upload 3 blobs to dict_of_blobs with 25, 35 and 45 bytes respectively
        self.controller.upload_blob(
            self.req, 'sample_artifact', art1['id'], 'dict_of_blobs/blob1',
            BytesIO(b'a' * 25), 'application/octet-stream')
        self.controller.upload_blob(
            self.req, 'sample_artifact', art1['id'], 'dict_of_blobs/blob2',
            BytesIO(b'a' * 35), 'application/octet-stream')
        self.controller.upload_blob(
            self.req, 'sample_artifact', art1['id'], 'dict_of_blobs/blob3',
            BytesIO(b'a' * 45), 'application/octet-stream')
        self.assertEqual(115, api.calculate_uploaded_data(
            self.req.context, self.session))

        # create another sample artifact and upload 100 bytes there
        art2 = self.controller.create(
            self.req, 'sample_artifact', {'name': 'art2'})
        self.controller.upload_blob(
            self.req, 'sample_artifact', art2['id'], 'blob',
            BytesIO(b'a' * 100), 'application/octet-stream')
        self.assertEqual(215, api.calculate_uploaded_data(
            self.req.context, self.session))

        # create image and upload 150 bytes there
        img1 = self.controller.create(
            self.req, 'images', {'name': 'img1'})
        self.controller.upload_blob(
            self.req, 'images', img1['id'], 'image',
            BytesIO(b'a' * 150), 'application/octet-stream')
        # the whole amount of uploaded data is 365 bytes
        self.assertEqual(365, api.calculate_uploaded_data(
            self.req.context, self.session))
        # 215 bytes for sample_artifact
        self.assertEqual(215, api.calculate_uploaded_data(
            self.req.context, self.session, 'sample_artifact'))
        # 150 bytes for images
        self.assertEqual(150, api.calculate_uploaded_data(
            self.req.context, self.session, 'images'))

        # create an artifact from another user and check that it's not included
        # for the original user
        req = self.get_fake_request(self.users['user2'])
        another_art = self.controller.create(
            req, 'sample_artifact', {'name': 'another'})
        # upload 1000 bytes to 'blob'
        self.controller.upload_blob(
            req, 'sample_artifact', another_art['id'], 'blob',
            BytesIO(b'a' * 1000), 'application/octet-stream')

        # original user still has 365 bytes
        self.assertEqual(365, api.calculate_uploaded_data(
            self.req.context, self.session))
        # user2 has 1000
        self.assertEqual(
            1000, api.calculate_uploaded_data(req.context, self.session))

    def test_quota_operations(self):
        # create several quotas
        values = {
            "project1": {
                "max_uploaded_data": 1000,
                "max_uploaded_data:images": 500,
                "max_artifact_number": 10
            },
            "project2": {
                "max_uploaded_data": 1000,
                "max_uploaded_data:sample_artifact": 500,
                "max_artifact_number": 20
            },
            "project3": {
                "max_uploaded_data": 1000
            }
        }

        api.set_quotas(values, self.session)

        res = api.get_all_quotas(self.session)
        self.assertEqual(values, res)

        # Redefine quotas
        new_values = {
            "project1": {
                "max_uploaded_data": 200,
                "max_uploaded_data:images": 1000,
                "max_artifact_number": 30,
                "max_artifact_number:images": 20
            },
            "project2": {},
        }

        api.set_quotas(new_values, self.session)

        # project3 should remain unchanged
        new_values['project3'] = {"max_uploaded_data": 1000}
        # project 2 quotas removed
        new_values.pop('project2')

        res = api.get_all_quotas(self.session)
        self.assertEqual(new_values, res)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from six import BytesIO

from glare.common import exception as exc
from glare.db import artifact_api
from glare.tests.unit import base


class TestArtifactDownload(base.BaseTestArtifactAPI):
    def setUp(self):
        super(TestArtifactDownload, self).setUp()
        values = {'name': 'ttt', 'version': '1.0', 'string_required': 'str2'}
        self.sample_artifact = self.controller.create(
            self.req, 'sample_artifact', values)

        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')

        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['blob']['size'])
        self.assertEqual('active', artifact['blob']['status'])

    def test_download_basic(self):
        downloaded_blob = self.controller.download_blob(
            self.req, 'sample_artifact',
            self.sample_artifact['id'], 'blob')
        self.assertEqual(b'aaa', downloaded_blob['data'].data)

    def test_download_from_folders(self):
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/folder1',
            BytesIO(b'bbb'), 'application/octet-stream')
        downloaded_blob = self.controller.download_blob(
            self.req, 'sample_artifact',
            self.sample_artifact['id'], 'dict_of_blobs/folder1')
        self.assertEqual(b'bbb', downloaded_blob['data'].data)

        # Negative dict_of_blobs tests:
        # Key error
        self.assertRaises(exc.NotFound, self.controller.download_blob,
                          self.req, 'sample_artifact',
                          self.sample_artifact['id'],
                          "dict_of_blobs/ImaginaryFolder")

        # incorrect dict_of_blobs spelling
        self.assertRaises(exc.BadRequest, self.controller.download_blob,
                          self.req, 'sample_artifact',
                          self.sample_artifact['id'],
                          "NOT_DICT_FIELD/folder1")

    def test_download_from_non_existing_fields(self):
        self.assertRaises(exc.BadRequest, self.controller.download_blob,
                          self.req, 'sample_artifact',
                          self.sample_artifact['id'], "NON_EXISTING_FIELD")

    def test_download_of_saving_blob(self):
        self.sample_artifact = self.controller.show(
            self.req, 'sample_artifact', self.sample_artifact['id'])

        # Change status of the blob to 'saving'
        self.sample_artifact['blob']['status'] = 'saving'
        artifact_api.ArtifactAPI().update_blob(
            self.req.context, self.sample_artifact['id'],
            {'blob': self.sample_artifact['blob']})

        self.sample_artifact = self.controller.show(
            self.req, 'sample_artifact', self.sample_artifact['id'])
        self.assertEqual('saving', self.sample_artifact['blob']['status'])

        # assert that we can't download while blob in saving status
        self.assertRaises(exc.Conflict, self.controller.download_blob,
                          self.req, 'sample_artifact',
                          self.sample_artifact['id'], "blob")

    def test_download_from_deactivated_artifact_as_other_user(self):
        self.req = self.get_fake_request(user=self.users['admin'])
        art = self.controller.show(self.req, 'sample_artifact',
                                   self.sample_artifact['id'])
        # change artifact status to deactivted: draft-> activate -> deactivated
        for status in ['active', 'deactivated']:
            changes = [{'op': 'replace', 'path': '/status', 'value': status}]
            self.req = self.get_fake_request(user=self.users['admin'])
            art = self.update_with_values(changes, art_id=art['id'])

        # make request from other user (That didn't create the artifact)
        self.req = self.get_fake_request(user=self.users['user1'])
        self.assertRaises(exc.Forbidden, self.controller.download_blob,
                          self.req, 'sample_artifact',
                          art['id'], "blob")
        # Make sure that admin can download from deactivated artifact
        self.req = self.get_fake_request(user=self.users['admin'])
        downloaded_blob = self.controller.download_blob(
            self.req, 'sample_artifact', art['id'], 'blob')
        self.assertEqual(b'aaa', downloaded_blob['data'].data)

    def test_download_for_deleted_artifact(self):
        self.config(delayed_delete=True)
        self.controller.delete(self.req, 'sample_artifact',
                               self.sample_artifact['id'])
        self.assertRaises(exc.Forbidden, self.controller.download_blob,
                          self.req, 'sample_artifact',
                          self.sample_artifact['id'], "blob")

    def test_download_external_blob(self):
        values = {'name': 'aaa', 'version': '2.0'}
        url = "http: // FAKE_LOCATION.COM"
        content_type = 'application/vnd+openstack.glare-custom-location+json'
        art = self.controller.create(self.req, 'sample_artifact', values)
        body = {'url': url, 'md5': "fake"}
        self.controller.upload_blob(self.req, 'sample_artifact', art['id'],
                                    'blob', body, content_type)
        downloaded_blob = self.controller.download_blob(self.req,
                                                        'sample_artifact',
                                                        art['id'], 'blob')

        self.assertEqual(url, downloaded_blob['data']['url'])
        self.assertTrue(downloaded_blob['meta']['external'])
        self.assertEqual("fake", downloaded_blob['meta']['md5'])
        self.assertIsNone(downloaded_blob['meta']['sha1'])
        self.assertIsNone(downloaded_blob['meta']['sha256'])



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import jsonschema
from jsonschema import exceptions as json_exceptions
from oslo_versionedobjects import fields
import semantic_version
import six
import six.moves.urllib.parse as urlparse
import six.moves.urllib.request as urlrequest

from glare.common import exception
from glare.i18n import _


class Version(fields.FieldType):

    @staticmethod
    def coerce(obj, field, value):
        return str(semantic_version.Version.coerce(str(value)))


class VersionField(fields.AutoTypedField):
    AUTO_TYPE = Version()


class BlobFieldType(fields.FieldType):
    """Blob field contains reference to blob location.
    """
    BLOB_STATUS = (SAVING, ACTIVE) = ('saving', 'active')

    BLOB_SCHEMA = {
        'type': 'object',
        'properties': {
            'url': {'type': ['string', 'null'], 'format': 'uri',
                    'maxLength': 2048},
            'size': {'type': ['number', 'null']},
            'md5': {'type': ['string', 'null']},
            'sha1': {'type': ['string', 'null']},
            'sha256': {'type': ['string', 'null']},
            'external': {'type': 'boolean'},
            'id': {'type': 'string'},
            'status': {'type': 'string',
                       'enum': list(BLOB_STATUS)},
            'content_type': {'type': ['string', 'null']},
        },
        'required': ['url', 'size', 'md5', 'sha1', 'sha256', 'external',
                     'status', 'id', 'content_type']
    }

    @staticmethod
    def coerce(obj, field, value):
        """Validate and store blob info inside oslo.vo"""
        if not isinstance(value, dict):
            raise ValueError(_("Blob value must be dict. Got %s type instead")
                             % type(value))
        try:
            jsonschema.validate(value, BlobFieldType.BLOB_SCHEMA)
        except json_exceptions.ValidationError as e:
            raise ValueError(e)

        return value

    @staticmethod
    def to_primitive(obj, field, value):
        prim = {key: val for key, val in value.items()
                if key != 'id'}

        if not value.get('external'):
            url = '/artifacts/%(name)s/%(id)s/' % {
                "name": obj.get_type_name(),
                'id': obj.id
            }
            blob_path = field.split('[')
            url = url + blob_path[0]
            if len(blob_path) > 1:
                url += '/%s' % blob_path[1][1:-2]
            prim['url'] = url
        return prim


class BlobField(fields.AutoTypedField):
    AUTO_TYPE = BlobFieldType()


class LinkFieldType(fields.FieldType):
    """Link field specifies Artifact dependency on other artifact or some
    external resource. From technical perspective it is just soft link to Glare
    Artifact or https/http resource. So Artifact users can download the
    referenced file by that link.
    """

    @staticmethod
    def is_external(link):
        return link.startswith('http')

    @staticmethod
    def get_type_name(link):
        url = link.split('/')
        if len(url) == 4:
            return url[2]
        else:
            raise ValueError(_("It is not possible to "
                               "extract type_name from link %s"), link)

    @staticmethod
    def coerce(obj, field, value):
        # to remove the existing link user sets its value to None,
        # we have to consider this case.
        if value is None:
            return value
        # check that value is string
        if not isinstance(value, six.string_types):
            raise ValueError(_('A string is required in field %(field)s, '
                               'not a %(type)s') %
                             {'field': field, 'type': type(value).__name__})
        # determine if link is external or internal
        external = LinkFieldType.is_external(value)
        # validate link itself
        if external:
            link = urlparse.urlparse(value)
            if link.scheme not in ('http', 'https'):
                raise ValueError(_('Only http and https requests '
                                   'are allowed in url %s') % value)
            try:
                with urlrequest.urlopen(value) as data:
                    data.read(1)
            except Exception:
                raise ValueError(
                    _('Link %(link)s is not valid in field '
                      '%(field)s. The link must be either valid url or '
                      'reference to artifact. Example: '
                      'http://glarehost:9494/artifacts/<artifact_type>/'
                      '<artifact_id>'
                      ) % {'link': value, 'field': field})
        else:
            result = value.split('/')
            if len(result) != 4 or result[1] != 'artifacts':
                raise ValueError(
                    _('Link %(link)s is not valid in field '
                      '%(field)s. The link must be either valid url or '
                      'reference to artifact. Example: '
                      '/artifacts/<artifact_type>/<artifact_id>'
                      ) % {'link': value, 'field': field})
            # try to find the referenced artifact
            try:
                obj.db_api.get(obj.obj_context, result[3])
            except exception.NotFound:
                raise ValueError(
                    _("Link %(link)s is not valid in field %(field)s, because "
                      "artifact with id %(art_id)s doesn't exist"
                      ) % {'link': value, 'field': field, 'art_id': result[3]}
                )

        return value


class Link(fields.AutoTypedField):
    AUTO_TYPE = LinkFieldType()


class List(fields.AutoTypedField):

    def __init__(self, element_type, **kwargs):
        self.AUTO_TYPE = fields.List(element_type())
        super(List, self).__init__(**kwargs)


class Dict(fields.AutoTypedField):

    def __init__(self, element_type, **kwargs):
        self.AUTO_TYPE = fields.Dict(element_type())
        super(Dict, self).__init__(**kwargs)
